(* jast.ml *)
(* An abstract syntax tree representing Java constructs. More or less generic. *)
(* @authors Team GRAPL *)

type op = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq | Not | Inc | Dec | Noop
  
type expr = 
	| Literal of float
	| BooleanLiteral of string
  | Id of string 
  | Binop of expr*op*expr
	| Unop of op*expr
  | Assign of string*expr
  | Call of expr*string*expr list (* implicit-param * function-name * arg-list *)
  | Noexpr 
  | List of expr list
  | StringLit of string
    		
type stmt =
  | New of string*string*expr list (* e.g. n = new Node(); *)
	| NewDecl of string*string*expr list (* e.g. Node n = new Node(); *)
	| Decl of string*string (* e.g. number x *)
	| Block of stmt list  	(* Statements *)
  | Expr of expr			(* { ... } *)
  | Foreach of string*string*expr*stmt (* e.g. for (Node n : graph.getNode("a", "<", 6, "visited")) { stmt } *)
	| While of expr*stmt (* predicate * body *)
  | IfThenElse of expr*stmt*stmt (* e.g. if (foo == 42) then {} else {} *)
	| Return of expr
	| Nostmt
  
type method_decl = {
  	jrettype: string;
    jfname : string;
    jformals : (string*string) list;
    jlocals : (string*string) list;
    jbody : stmt list;
  }

(* class-name * globals-list * method-decls-list) *)
type java_class = string * string list * method_decl list
  
(* imports-list * class-list *)	
type program = string list * java_class list

