(* scanner.mll *)
(* @authors: Steven Edwards, Lili Chen, Di Wen *)

{ open Parser }

(* letter, digit, and definition of number in GRAPL *)
let letter = ['a'-'z' 'A'-'Z']
let digit = ['0'-'9']
let num = digit+ | digit*'.'digit+

rule token = parse
| "/*"    { comment lexbuf }
| [' ' '\t' '\r' '\n'] { token lexbuf }
| '(' { LPAREN }   
| ')' { RPAREN }
| '{' { LBRACE }
| '}' { RBRACE }
| '[' { LBRACK }
| ']' { RBRACK }
| "!=" { NEQ }  
| '>'  { GT }
| '<'  { LT }
| "<=" { LEQ }
| ">=" { GEQ }
| "==" { EQ }
| '='  { ASSIGN }
| '!'  { NOT } 
| "++" { INC }
| "--" { DEC }
| "<<"  { LEDGE }
| ">>"  { REDGE }
| "<>"  { BEDGE }
| "::"  { CONS }
| "while" { WHILE }
| "graph" { GRAPH }
| "node"         { NODE }
| "list"         { LIST }
| "number"         { NUMBER }
| "boolean"         { BOOLEAN }
| "bool"         { BOOLEAN }
| "void"         { VOID }
| "if"           { IF }
| "then"         { THEN }
| "else"         { ELSE }
| "forEach"      { FOREACH }
| "foreach"      { FOREACH }
| "from"         { FROM }
| "to"           { TO }
| "with"         { WITH }
| "return"       { RETURN }
| "visited"        { VISITED }
| "visit"          { VISIT }
| "unvisit"        { UNVISIT }
| "isVisited"      { ISVISITED }
| "unVisited"      { UNVISITED }
| "unvisited"      { UNVISITED }
| ';' { SEMICOLON }
| ',' { COMMA }
| '+' { PLUS }
| '-' { MINUS }
| '*' { TIMES }
| '/' { DIVIDE }
| "&&" { AND }
| "||" { OR }
| num as s { LITERAL(float_of_string(s)) }
| letter (letter | digit | '_')* as identi { ID(identi) }
| eof { EOF }
| "true" { BOOLEAN_LITERAL("true") }
| "false" { BOOLEAN_LITERAL("false") } 

(* comment *)
and comment = parse
  "*/" { token lexbuf }
| _ { comment lexbuf }
